@@ -28,7 +28,11 @@ from distributed.comm.utils import (
     host_array,
     to_frames,
 )
-from distributed.diagnostics.nvml import has_cuda_context
+from distributed.diagnostics.nvml import (
+    CudaDeviceInfo,
+    get_device_index_and_uuid,
+    has_cuda_context,
+)
 from distributed.utils import ensure_ip, get_ip, get_ipv6, log_errors, nbytes
 
 logger = logging.getLogger(__name__)
@@ -57,17 +61,27 @@ _warning_suffix = (
 )
 
 
-def _warn_existing_cuda_context(ctx, pid):
+def _get_device_and_uuid_str(device_info: CudaDeviceInfo) -> str:
+    return f"{device_info.device_index} ({str(device_info.uuid)})"
+
+
+def _warn_existing_cuda_context(device_info: CudaDeviceInfo, pid: int) -> None:
+    device_uuid_str = _get_device_and_uuid_str(device_info)
     logger.warning(
-        f"A CUDA context for device {ctx} already exists on process ID {pid}. {_warning_suffix}"
+        f"A CUDA context for device {device_uuid_str} already exists "
+        f"on process ID {pid}. {_warning_suffix}"
     )
 
 
-def _warn_cuda_context_wrong_device(ctx_expected, ctx_actual, pid):
+def _warn_cuda_context_wrong_device(
+    device_info_expected: CudaDeviceInfo, device_info_actual: CudaDeviceInfo, pid: int
+) -> None:
+    expected_device_uuid_str = _get_device_and_uuid_str(device_info_expected)
+    actual_device_uuid_str = _get_device_and_uuid_str(device_info_actual)
     logger.warning(
         f"Worker with process ID {pid} should have a CUDA context assigned to device "
-        f"{ctx_expected}, but instead the CUDA context is on device {ctx_actual}. "
-        f"{_warning_suffix}"
+        f"{expected_device_uuid_str}, but instead the CUDA context is on device "
+        f"{actual_device_uuid_str}. {_warning_suffix}"
     )
 
 
@@ -116,22 +130,24 @@ def init_once():
                 "CUDA support with UCX requires Numba for context management"
             )
 
-        cuda_visible_device = int(
+        cuda_visible_device = get_device_index_and_uuid(
             os.environ.get("CUDA_VISIBLE_DEVICES", "0").split(",")[0]
         )
         pre_existing_cuda_context = has_cuda_context()
-        if pre_existing_cuda_context is not False:
-            _warn_existing_cuda_context(pre_existing_cuda_context, os.getpid())
+        if pre_existing_cuda_context.has_context:
+            _warn_existing_cuda_context(
+                pre_existing_cuda_context.device_info, os.getpid()
+            )
 
         numba.cuda.current_context()
 
         cuda_context_created = has_cuda_context()
         if (
-            cuda_context_created is not False
-            and cuda_context_created != cuda_visible_device
+            cuda_context_created.has_context
+            and cuda_context_created.device_info.uuid != cuda_visible_device.uuid
         ):
             _warn_cuda_context_wrong_device(
-                cuda_visible_device, cuda_context_created, os.getpid()
+                cuda_visible_device, cuda_context_created.device_info, os.getpid()
             )
 
     import ucp as _ucp
@@ -316,7 +332,7 @@ class UCX(Comm):
             for each_frame in send_frames:
                 await self.ep.send(each_frame)
             return sum(sizes)
-        except (ucp.exceptions.UCXBaseException):
+        except ucp.exceptions.UCXBaseException:
             self.abort()
             raise CommClosedError("While writing, the connection was closed")
 
@@ -400,7 +416,7 @@ class UCX(Comm):
         if self._ep is not None:
             try:
                 await self.ep.send(struct.pack("?Q", True, 0))
-            except (
+            except (  # noqa: B030
                 ucp.exceptions.UCXError,
                 ucp.exceptions.UCXCloseError,
                 ucp.exceptions.UCXCanceled,
