@@ -17,8 +17,8 @@ from distributed.dashboard.components.scheduler import (
     BandwidthWorkers,
     ClusterMemory,
     ComputePerKey,
+    Contention,
     CurrentLoad,
-    EventLoop,
     ExceptionsTable,
     MemoryByKey,
     Occupancy,
@@ -110,7 +110,7 @@ applications = {
     "/individual-compute-time-per-key": individual_doc(ComputePerKey, 500),
     "/individual-aggregate-time-per-action": individual_doc(AggregateAction, 500),
     "/individual-scheduler-system": individual_doc(SystemMonitor, 500),
-    "/individual-event-loop": individual_doc(EventLoop, 500),
+    "/individual-contention": individual_doc(Contention, 500),
     "/individual-profile": individual_profile_doc,
     "/individual-profile-server": individual_profile_server_doc,
     "/individual-gpu-memory": gpu_memory_doc,
