@@ -175,7 +175,9 @@ class Cluster(SyncMethodMixin):
                         exc_info=True,
                     )
             # Sleep, with error backoff
-            interval = min(max_interval, self._sync_interval * 1.5**err_count)
+            interval = _exponential_backoff(
+                err_count, self._sync_interval, 1.5, max_interval
+            )
             await asyncio.sleep(interval)
 
     async def _close(self):
@@ -579,3 +581,15 @@ class Cluster(SyncMethodMixin):
 
     def __hash__(self):
         return id(self)
+
+
+def _exponential_backoff(
+    attempt: int, multiplier: float, exponential_base: float, max_interval: float
+) -> float:
+    """Calculate the duration of an exponential backoff"""
+    try:
+        interval = multiplier * exponential_base**attempt
+    except OverflowError:
+        return max_interval
+
+    return min(max_interval, interval)
