@@ -1,17 +1,17 @@
 from __future__ import annotations
 
 import logging
-from collections.abc import Iterator, Mapping, MutableMapping, Sized
+from collections import defaultdict
+from collections.abc import Hashable, Iterator, Mapping, MutableMapping, Sized
 from contextlib import contextmanager
-from dataclasses import dataclass
 from functools import partial
-from time import perf_counter
 from typing import Any, Literal, NamedTuple, Protocol, cast
 
 from packaging.version import parse as parse_version
 
 import zict
 
+from distributed.metrics import context_meter
 from distributed.protocol import deserialize_bytes, serialize_bytelist
 from distributed.sizeof import safe_sizeof
 from distributed.utils import RateLimiterFilter
@@ -69,49 +69,6 @@ class ManualEvictProto(Protocol):
         ...  # pragma: nocover
 
 
-@dataclass
-class FastMetrics:
-    """Cumulative metrics for SpillBuffer.fast since the latest worker restart"""
-
-    read_count_total: int = 0
-    read_bytes_total: int = 0
-
-    def log_read(self, key_bytes: int) -> None:
-        self.read_count_total += 1
-        self.read_bytes_total += key_bytes
-
-
-@dataclass
-class SlowMetrics:
-    """Cumulative metrics for SpillBuffer.slow since the latest worker restart"""
-
-    read_count_total: int = 0
-    read_bytes_total: int = 0
-    read_time_total: float = 0
-    write_count_total: int = 0
-    write_bytes_total: int = 0
-    write_time_total: float = 0
-    pickle_time_total: float = 0
-    unpickle_time_total: float = 0
-
-    def log_write(
-        self,
-        key_bytes: int,
-        pickle_time: float,
-        write_time: float,
-    ) -> None:
-        self.write_count_total += 1
-        self.write_bytes_total += key_bytes
-        self.pickle_time_total += pickle_time
-        self.write_time_total += write_time
-
-    def log_read(self, key_bytes: int, read_time: float, unpickle_time: float) -> None:
-        self.read_count_total += 1
-        self.read_bytes_total += key_bytes
-        self.read_time_total += read_time
-        self.unpickle_time_total += unpickle_time
-
-
 # zict.Buffer[str, Any] requires zict >= 2.2.0
 class SpillBuffer(zict.Buffer):
     """MutableMapping that automatically spills out dask key/value pairs to disk when
@@ -129,7 +86,8 @@ class SpillBuffer(zict.Buffer):
     """
 
     logged_pickle_errors: set[str]
-    fast_metrics: FastMetrics
+    #: (label, unit) -> ever-increasing cumulative value
+    cumulative_metrics: defaultdict[tuple[str, str], float]
 
     def __init__(
         self,
@@ -145,10 +103,28 @@ class SpillBuffer(zict.Buffer):
 
         super().__init__(fast={}, slow=slow, n=target, weight=_in_memory_weight)
         self.logged_pickle_errors = set()  # keys logged with pickle error
-        self.fast_metrics = FastMetrics()
+        self.cumulative_metrics = defaultdict(float)
 
     @contextmanager
-    def handle_errors(self, key: str | None) -> Iterator[None]:
+    def _capture_metrics(self) -> Iterator[None]:
+        """Capture metrics re. disk read/write, serialize/deserialize, and
+        compress/decompress.
+
+        Note that this duplicates capturing from gather_dep, get_data, and execute. It
+        is repeated here to make it possible to split serialize/deserialize and
+        compress/decompress triggered by spill/unspill from those triggered by network
+        comms.
+        """
+
+        def metrics_callback(label: Hashable, value: float, unit: str) -> None:
+            assert isinstance(label, str)
+            self.cumulative_metrics[label, unit] += value
+
+        with context_meter.add_callback(metrics_callback):
+            yield
+
+    @contextmanager
+    def _handle_errors(self, key: str | None) -> Iterator[None]:
         try:
             yield
         except MaxSpillExceeded as e:
@@ -211,7 +187,7 @@ class SpillBuffer(zict.Buffer):
         issue remained in fast.
         """
         try:
-            with self.handle_errors(key):
+            with self._capture_metrics(), self._handle_errors(key):
                 super().__setitem__(key, value)
                 self.logged_pickle_errors.discard(key)
         except HandledError:
@@ -229,20 +205,25 @@ class SpillBuffer(zict.Buffer):
         This method never raises.
         """
         try:
-            with self.handle_errors(None):
+            with self._capture_metrics(), self._handle_errors(None):
                 _, _, weight = self.fast.evict()
                 return cast(int, weight)
         except HandledError:
             return -1
 
     def __getitem__(self, key: str) -> Any:
-        if key in self.fast:
-            # Note: don't log from self.fast.__getitem__, because that's called every
-            # time a key is evicted, and we don't want to count those events here.
-            nbytes = cast(int, self.fast.weights[key])
-            self.fast_metrics.log_read(nbytes)
+        with self._capture_metrics():
+            if key in self.fast:
+                # Note: don't log from self.fast.__getitem__, because that's called
+                # every time a key is evicted, and we don't want to count those events
+                # here.
+                nbytes = cast(int, self.fast.weights[key])
+                # This is logged not only by the internal metrics callback but also by
+                # those installed by gather_dep, get_data, and execute
+                context_meter.digest_metric("memory-read", 1, "count")
+                context_meter.digest_metric("memory-read", nbytes, "bytes")
 
-        return super().__getitem__(key)
+            return super().__getitem__(key)
 
     def __delitem__(self, key: str) -> None:
         super().__delitem__(key)
@@ -285,45 +266,6 @@ class SpillBuffer(zict.Buffer):
         """
         return self._slow_uncached.total_weight
 
-    def get_metrics(self) -> dict[str, float]:
-        """Metrics to be exported to Prometheus or to be parsed directly.
-
-        From these you may generate derived metrics:
-
-        cache hit ratio:
-          by keys  = memory_read_count_total / (memory_read_count_total + disk_read_count_total)
-          by bytes = memory_read_bytes_total / (memory_read_bytes_total + disk_read_bytes_total)
-
-        mean times per key:
-          pickle   = pickle_time_total     / disk_write_count_total
-          write    = disk_write_time_total / disk_write_count_total
-          unpickle = unpickle_time_total   / disk_read_count_total
-          read     = disk_read_time_total  / disk_read_count_total
-
-        mean bytes per key:
-          write    = disk_write_bytes_total / disk_write_count_total
-          read     = disk_read_bytes_total  / disk_read_count_total
-
-        mean bytes per second:
-          write    = disk_write_bytes_total / disk_write_time_total
-          read     = disk_read_bytes_total  / disk_read_time_total
-        """
-        fm = self.fast_metrics
-        sm = self._slow_uncached.metrics
-
-        out = {
-            "memory_count": len(self.fast),
-            "memory_bytes": self.fast.total_weight,
-            "disk_count": len(self.slow),
-            "disk_bytes": self._slow_uncached.total_weight.disk,
-        }
-
-        for k, v in fm.__dict__.items():
-            out[f"memory_{k}"] = v
-        for k, v in sm.__dict__.items():
-            out[k if "pickle" in k else f"disk_{k}"] = v
-        return out
-
 
 def _in_memory_weight(key: str, value: Any) -> int:
     return safe_sizeof(value)
@@ -347,7 +289,6 @@ class Slow(zict.Func):
     max_weight: int | Literal[False]
     weight_by_key: dict[str, SpilledSize]
     total_weight: SpilledSize
-    metrics: SlowMetrics
 
     def __init__(self, spill_directory: str, max_weight: int | Literal[False] = False):
         super().__init__(
@@ -358,27 +299,16 @@ class Slow(zict.Func):
         self.max_weight = max_weight
         self.weight_by_key = {}
         self.total_weight = SpilledSize(0, 0)
-        self.metrics = SlowMetrics()
 
     def __getitem__(self, key: str) -> Any:
-        t0 = perf_counter()
-        pickled = self.d[key]
-        assert isinstance(pickled, bytearray if has_zict_230 else bytes)
-        t1 = perf_counter()
+        with context_meter.meter("disk-read", "seconds"):
+            pickled = self.d[key]
+        context_meter.digest_metric("disk-read", 1, "count")
+        context_meter.digest_metric("disk-read", len(pickled), "bytes")
         out = self.load(pickled)
-        t2 = perf_counter()
-
-        # For the sake of simplicity, we're not metering failure use cases.
-        self.metrics.log_read(
-            key_bytes=len(pickled),
-            read_time=t1 - t0,
-            unpickle_time=t2 - t1,
-        )
-
         return out
 
     def __setitem__(self, key: str, value: Any) -> None:
-        t0 = perf_counter()
         try:
             pickled = self.dump(value)
         except Exception as e:
@@ -391,7 +321,6 @@ class Slow(zict.Func):
             frame.nbytes if isinstance(frame, memoryview) else len(frame)
             for frame in pickled
         )
-        t1 = perf_counter()
 
         # Thanks to Buffer.__setitem__, we never update existing
         # keys in slow, but always delete them and reinsert them.
@@ -408,21 +337,15 @@ class Slow(zict.Func):
 
         # Store to disk through File.
         # This may raise OSError, which is caught by SpillBuffer above.
-        self.d[key] = pickled
-
-        t2 = perf_counter()
+        with context_meter.meter("disk-write", "seconds"):
+            self.d[key] = pickled
+        context_meter.digest_metric("disk-write", 1, "count")
+        context_meter.digest_metric("disk-write", pickled_size, "bytes")
 
         weight = SpilledSize(safe_sizeof(value), pickled_size)
         self.weight_by_key[key] = weight
         self.total_weight += weight
 
-        # For the sake of simplicity, we're not metering failure use cases.
-        self.metrics.log_write(
-            key_bytes=pickled_size,
-            pickle_time=t1 - t0,
-            write_time=t2 - t1,
-        )
-
     def __delitem__(self, key: str) -> None:
         super().__delitem__(key)
         self.total_weight -= self.weight_by_key.pop(key)
